import re
import json
import jsonpath


def load_har(har_path):
    """

    :param har_path:
    :return:
    """
    with open(har_path, 'r', encoding='utf-8') as har:
        var = json.loads(har.read())
        return jsonpath.jsonpath(var, '$.log.entries..request')


def save_postman_collection(file_path, postman_json):
    with open(file_path, 'w') as postman:
        postman.write(json.dumps(postman_json, indent=4, ensure_ascii=False))


def extract_path(url: str):
    if url.count('/') < 3 or (url.count('3') == 3 and url[-1] == '/'):
        # eg. https://www.baidu.com
        # eg. https://www.baidu.com/
        return []

    if url.find('?') != -1:
        url = url.split('?')[0]
    return re.search('//.+?/(.+)', url).group(1).split('/')


def extract_params(url):
    """
    :param url: (str) eg: https://www.baidu.com/wd?q=testerhome&encoding=utf-8
    :return:
        [
            {
                "key": "q",
                "value": "testerhome"
            },
            {
                 "key": "encoding",
                "value": "utf-8"
            }
        ]
    """
    if url.find('?') == -1:
        return []
    query = []
    query_list = url.split('?')[-1].split('&')
    for i in query_list:
        key, value = i.split('=')
        query.append({
            'key': key,
            'value': value
        })
    return query


def change_dict_key(har_dict):
    """

    :param har_dict: [
        {"name": "", "value": ""},
        ...
    ]
    :return: [
        {"key": "", "value": ""},
        ...
    ]
    """
    for header in har_dict:
        header['key'] = header.pop('name')
    return har_dict


def change_url(har_request):
    """
    :param har_request: {'url': 'https://www.baidu.com/s?wd=12345'}
    :return: {
        'protocol': https,
        'host': ['www', 'baidu', 'com'],
        'path': ['s'],
        'query': [
            {'key': 'wd', 'value': '12345'},
            ...
        ]
    }
    """

    url_tmp: str = har_request['url']

    # 提取协议 http || https
    protocol = re.search('(https*):', url_tmp).group(1)

    # 提取host
    host = re.search('://(.+[^/])/?', url_tmp).group(1).split('.')

    # 提取path
    path = extract_path(url_tmp)

    # 提取query
    query = extract_params(url_tmp)

    return {
        'protocol': protocol,
        'host': host,
        'path': path,
        'query': query
    }


def change_body(har_request):
    mime_type: str = jsonpath.jsonpath(har_request, '$.postData.mimeType')[0]

    if mime_type is None:
        return None

    if mime_type.find('form-data') != -1:
        return {'mode': 'formdata', 'formdata': change_dict_key(har_request['postData']['params'])}

    elif mime_type.find('x-www-form-urlencoded') != -1:
        return {'mode': 'urlencoded', 'urlencoded': change_dict_key(har_request['postData']['params'])}

    elif mime_type.find('json') != -1:
        return {
            'mode': 'raw',
            'raw': har_request['postData']['text']
        }

    else:
        raise Exception('无法识别:%s' % mime_type)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['har2postman']

package_data = \
{'': ['*']}

install_requires = \
['jsonpath>=0.82.0,<0.83.0']

entry_points = \
{'console_scripts': ['harto = har2postman.cli:main']}

setup_kwargs = {
    'name': 'har2postman',
    'version': '0.1.1',
    'description': 'Convert HAR(HTTP Archive) to Postman Collection',
    'long_description': '## Har2Postman\n[![travis-ci](https://api.travis-ci.org/whitexie/Har2Postman.svg?branch=master)](https://travis-ci.org/whitexie/Har2toPostman)\n![coveralls](https://coveralls.io/repos/github/whitexie/Har2Postman/badge.svg?branch=master)\n> 将har文件转换为postman可导入文件\n\n## 安装\n```shell script\npip isntall Har2toPostman\n```\n\n## 使用\n1.将har文件转换为postman可导入文件\n```shell script\nharto postman_echo.har\n\n# INFO:root:read postman_echo.har\n# INFO:root:Generate postman collection successfully: postman_echo.json\n```\n2.在postman中导入postman_echo.json文件\n![](https://github.com/whitexie/Har2Postman/blob/master/postman_collection1.png)\n',
    'author': 'ysansan',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/whitexie/Har2Postman',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
